@@ -317,6 +317,8 @@ typedef enum {
 	RESPONSE_CONTROL_STATUS,
 	REQUEST_BURST_BUFFER_STATUS,
 	RESPONSE_BURST_BUFFER_STATUS,
+	REQUEST_JOB_STATE,
+	RESPONSE_JOB_STATE,
 
 	REQUEST_CRONTAB = 2200,
 	RESPONSE_CRONTAB,
@@ -418,6 +420,8 @@ typedef enum {
 	REQUEST_TOP_JOB,		/* 5038 */
 	REQUEST_AUTH_TOKEN,
 	RESPONSE_AUTH_TOKEN,
+	REQUEST_KILL_JOBS, /* 5041 */
+	RESPONSE_KILL_JOBS,
 
 	REQUEST_LAUNCH_TASKS = 6001,
 	RESPONSE_LAUNCH_TASKS,
@@ -701,6 +705,11 @@ typedef struct job_info_request_msg {
 				 * jobs. */
 } job_info_request_msg_t;
 
+typedef struct {
+	uint32_t count;
+	slurm_selected_step_t *job_ids;
+} job_state_request_msg_t;
+
 typedef struct {
 	uint16_t show_flags;
 	char *container_id;
@@ -1073,13 +1082,11 @@ typedef struct kill_job_msg {
 } kill_job_msg_t;
 
 typedef struct reattach_tasks_request_msg {
+	char *io_key;
 	uint16_t     num_resp_port;
 	uint16_t    *resp_port; /* array of available response ports */
 	uint16_t     num_io_port;
 	uint16_t    *io_port;   /* array of available client IO ports */
-	slurm_cred_t *cred;      /* used only a weak authentication mechanism
-				   for the slurmstepd to use when connecting
-				   back to the client */
 	slurm_step_id_t step_id;
 } reattach_tasks_request_msg_t;
 
@@ -1589,6 +1596,31 @@ extern char **slurm_char_array_copy(int n, char **src);
  */
 extern char *slurm_sort_node_list_str(char *node_list);
 
+/*
+ * For each token in a comma delimited job array expression set the matching
+ * bitmap entry.
+ *
+ * IN tok - An array expression. For example: "[1,3-5,8]"
+ * IN array_bitmap - Matching entries in tok are set in this bitmap.
+ * IN max - maximum size of the job array.
+ * RET true if tok is a valid array expression, v
+ */
+extern bool slurm_parse_array_tok(char *tok, bitstr_t *array_bitmap,
+				  uint32_t max);
+/*
+ * Take a string representation of an array range or comma separated values
+ * and translate that into a bitmap.
+ *
+ * IN str - An array expression. For example: "[1,3-5,8]"
+ * IN max_array_size - maximum size of the job array.
+ * OUT i_last_p - (optional) if not NULL, set the value pointed to by i_last_p
+ *                to the last bit set in the array bitmap. This is only changed
+ *                if the string was successfully parsed.
+ * RET array_bitmap if successfull, NULL otherwise.
+ */
+extern bitstr_t *slurm_array_str2bitmap(char *str, uint32_t max_array_size,
+					int32_t *i_last_p);
+
 /*
  * Take a string identifing any part of a job and parses it into an id
  *
@@ -1609,9 +1641,12 @@ extern char *slurm_sort_node_list_str(char *node_list);
  * IN/OUT id - ptr to id to be populated.
  * 	All values are always set to NO_VAL and then populated as parsed.
  *      (Errors during parsing may result in partially populated ID.)
+ * IN max_array_size - Maximum size of a job array. May be 0 or NO_VAL if
+ *                     job array expressions are not expected.
  * RET SLURM_SUCCESS or error
  */
-extern int unfmt_job_id_string(const char *src, slurm_selected_step_t *id);
+extern int unfmt_job_id_string(const char *src, slurm_selected_step_t *id,
+			       uint32_t max_array_size);
 /*
  * Dump id into string identifing a part of a job.
  * Dumps same formats as unfmt_job_id_string() parsed.
@@ -1642,6 +1677,7 @@ extern void slurm_free_container_id_request_msg(
 extern void slurm_free_container_id_response_msg(
 	container_id_response_msg_t *msg);
 extern void slurm_free_job_info_request_msg(job_info_request_msg_t *msg);
+extern void slurm_free_job_state_request_msg(job_state_request_msg_t *msg);
 extern void slurm_free_job_step_info_request_msg(
 		job_step_info_request_msg_t *msg);
 extern void slurm_free_front_end_info_request_msg(
@@ -1733,6 +1769,9 @@ extern void slurm_free_reattach_tasks_response_msg(
 		reattach_tasks_response_msg_t * msg);
 extern void slurm_free_kill_job_msg(kill_job_msg_t * msg);
 extern void slurm_free_job_step_kill_msg(job_step_kill_msg_t * msg);
+extern void slurm_free_kill_jobs_msg(kill_jobs_msg_t *msg);
+extern void slurm_free_kill_jobs_resp_job_t(kill_jobs_resp_job_t *job_resp);
+extern void slurm_free_kill_jobs_response_msg(kill_jobs_resp_msg_t *msg);
 extern void slurm_free_epilog_complete_msg(epilog_complete_msg_t * msg);
 extern void slurm_free_srun_job_complete_msg(srun_job_complete_msg_t * msg);
 extern void slurm_free_srun_ping_msg(srun_ping_msg_t * msg);
@@ -1988,6 +2027,42 @@ extern void slurm_array16_to_value_reps(uint16_t *array, uint32_t array_cnt,
 extern int slurm_get_rep_count_inx(
 	uint32_t *rep_count, uint32_t rep_count_size, int inx);
 
+/*
+ * slurm_format_tres_string - given a TRES type and a tres-per-* string,
+ *			      will modifiy the string from the original
+ *			      colon-separated tres request format to the new
+ *			      '/' separating the TRES type from the tres
+ *			      request. This will work even if the request name
+ *			      or grestype with the TRES type.
+ * IN s - tres-per-* tres request string. This will be modified to fit the new
+ *	  format.
+ * IN tres_type - the type of tres to replace in the tres request string
+ *
+ * If *s is non-NULL, it must be an xmalloc'd string.
+ *
+ * Input:
+ * license:testing_license:3
+ * Output:
+ * license/testing_license:3
+ *
+ * This function will not modify correctly formatted tres request strings
+ * Input:
+ * license/testing_license:3
+ * Output:
+ * license/testing_license:3
+ *
+ * Input:
+ * gres:gres1:type1:3,gres:gres2:type2:6
+ * Output:
+ * gres:gres1/type1:3,gres/gres2:type2:6
+ *
+ * Input
+ * gres:gres1_gres:type1_gres:3,gres:gres2_gres:type2_gres:6
+ * Output:
+ * gres/gres1_gres:type1_gres:3,gres/gres2_gres:type2_gres:6
+ */
+extern void slurm_format_tres_string(char **s, char *tres_type);
+
 /*
  * Reentrant TRES specification parse logic
  * tres_type IN/OUT - type of tres we are looking for, If *tres_type is NULL we
