@@ -43,6 +43,7 @@
 
 /* getopt_long options, integers but not characters */
 #define OPT_LONG_NOCONVERT 0x100
+#define OPT_LONG_AUTOCOMP  0x101
 
 void _help_fields_msg(void);
 void _help_msg(void);
@@ -82,13 +83,14 @@ sstat [<OPTION>] -j <job(.stepid)>                                          \n\
      -j, --jobs:                                                            \n\
 	           Format is <job(.step)>. Stat this job step               \n\
                    or comma-separated list of job steps. This option is     \n\
-                   required.  The step portion will default to lowest step  \n\
-                   running if not specified, unless the --allsteps flag is  \n\
-                   set where not specifying a step will result in all       \n\
-                   running steps to be displayed.  A step id of 'batch'     \n\
-                   will display the information about the batch step. A     \n\
-                   step id of 'extern' will display the information about   \n\
-                   the extern step when using PrologFlags=contain.          \n\
+                   required.  The step portion will default to the lowest   \n\
+                   numbered (not batch, extern, etc) step running if not    \n\
+                   specified, unless the --allsteps flag is set where not   \n\
+                   specifying a step will result in all running steps       \n\
+                   to be displayed. A step id of 'batch' will display the   \n\
+                   information about the batch step. A step id of 'extern'  \n\
+                   will display the information about the extern step       \n\
+                   when using PrologFlags=contain.                          \n\
      -n, --noheader:                                                        \n\
 	           No header will be added to the beginning of output.      \n\
                    The default is to print a header.                        \n\
@@ -181,13 +183,14 @@ int decode_state_char(char *state)
 void parse_command_line(int argc, char **argv)
 {
 	extern int optind;
-	int c, i, optionIndex = 0;
+	int c, i, option_index = 0;
 	char *end = NULL, *start = NULL;
 	slurm_selected_step_t *selected_step = NULL;
 	ListIterator itr = NULL;
 	log_options_t logopt = LOG_OPTS_STDERR_ONLY;
 
 	static struct option long_options[] = {
+		{"autocomplete", required_argument, 0, OPT_LONG_AUTOCOMP},
 		{"allsteps", 0, 0, 'a'},
 		{"helpformat", 0, 0, 'e'},
 		{"help", 0, 0, 'h'},
@@ -212,7 +215,7 @@ void parse_command_line(int argc, char **argv)
 
 	while (1) {		/* now cycle through the command line */
 		c = getopt_long(argc, argv, "aehij:no:pPvV",
-				long_options, &optionIndex);
+				long_options, &option_index);
 		if (c == -1)
 			break;
 		switch (c) {
@@ -264,6 +267,10 @@ void parse_command_line(int argc, char **argv)
 			print_slurm_version();
 			exit(0);
 			break;
+		case OPT_LONG_AUTOCOMP:
+			suggest_completion(long_options, optarg);
+			exit(0);
+			break;
 		case ':':
 		case '?':	/* getopt() has explained it */
 			exit(1);
